/* -------------------------------------------------------------------
   Tutorial 7a : magnetic fields of an electromagnet

   Features:
   - Use of a template formulation library
   - Identical to Tutorial 2 for a static current source
   - Frequency-domain or time-domain solution for a time-dependent
     current source

   To compute the static solution in a terminal:
       getdp electromagnet -solve Magnetostatics2D_a -pos sta

   To compute the frequency-domain solution in a terminal:
       getdp electromagnet -solve Magnetodynamics2D_av -pos dyn

   To compute the time-dependent dynamic solution in a terminal:
       getdp electromagnet -setnumber TimeDomain 1 -solve Magnetodynamics2D_av -pos dyn

   To compute the solution interactively from the Gmsh GUI:
       File > Open > electromagnet.pro
       You may choose the Resolution in the left panel:
       Magnetodynamics2D_av (default) or Magnetostatics2D_a
       Run (button at the bottom of the left panel)
   ------------------------------------------------------------------- */

Include "electromagnet_common.pro";

Group {
  // Physical regions
  Air    = Region[ 101 ];   Core   = Region[ 102 ];
  Ind    = Region[ 103 ];   AirInf = Region[ 111 ];
  Surface_ht0 = Region[ 1100 ];
  Surface_bn0 = Region[ 1101 ];
  Surface_Inf = Region[ 1102 ];

  // Abstract regions used in the "Lib_Magnetodynamics2D_av_Cir.pro" template file
  // that is included below:
  Vol_Mag = Region[{Air, Core, Ind, AirInf}]; // full magnetic domain
  Vol_C_Mag = Region[Core]; // massive conductors
  Vol_S_Mag = Region[Ind]; // stranded conductors (coils)
  Vol_Inf_Mag = Region[AirInf]; // ring-shaped shell for infinite transformation
  Val_Rint = rInt; Val_Rext = rExt; // interior and exterior radii of ring

  DefineConstant[
    TimeDomain = {0, Choices{0 = "Frequency-domain", 1 = "Time-domain"},
      Name "Model parameters/03Analysis type"}
    NonLinearCore = {0, Choices{0, 1}, Visible TimeDomain,
      Name "Model parameters/Non linear B-H curve in core"}
  ];
  If(NonLinearCore)
    Vol_NL_Mag = Region[Core];
  EndIf
}

Function {
  DefineConstant[
    Current = {0.01, Name "Model parameters/Max. current [A]"},
    murCore = {100, Visible !NonLinearCore,
      Name "Model parameters/Mur core"}
    // by default, a linear ramp from 0 to 1 until 10 ms, then a constant value
    // of 1; any desired function can be specified
    timeFunction = {"($Time < TimeFinal / 2) ? (2 / TimeFinal * $Time) : 1",
      Choices{"($Time < TimeFinal / 2) ? (2 / TimeFinal * $Time) : 1",
        "Sin[2 * Pi * Freq * $Time]", "Sin[2 * Pi * 10 * $Time]"},
      Name "Model parameters/04Time function", Visible TimeDomain}

    // overwrite parameters from the "Lib_Magnetodynamics2D_av_Cir.pro"
    // template:
    Freq = {50, Visible !TimeDomain,
      Name "Model parameters/04Frequency [Hz]" }
    DeltaTime = {1e-3,
      Name "Model parameters/05Time step [s]", Visible TimeDomain}
    TimeFinal = {20e-3,
      Name "Model parameters/06Final simulation time [s]", Visible TimeDomain}
  ];

  Parse[StrCat["myModulation[] = ", timeFunction, ";"]];

  // Fix parameters from the "Lib_Magnetodynamics2D_av_Cir.pro" template:
  If(TimeDomain)
    Flag_FrequencyDomain = 0;
  Else
    Flag_FrequencyDomain = 1;
  EndIf

  mu0 = 4.e-7 * Pi;
  nu[ Region[{Air, Ind, AirInf}] ]  = 1. / mu0;

  If(NonLinearCore)
    Mat_core_b = {
      0, 0.0902935, 0.194131, 0.293454, 0.397291,
      0.496614, 0.600451, 0.690745, 0.794582, 0.893905,
      0.997743, 1.09707, 1.19639, 1.2912, 1.39955,
      1.49436, 1.54853, 1.59368, 1.63883, 1.693,
      1.74266, 1.79684, 1.8465, 1.89616};
    Mat_core_h = {
      0, 4.336661847, 7.34801661, 9.72476491, 11.5026052,
      13.1550314, 14.38338925, 15.72777105, 17.00435285, 17.9766304,
      19.00397556, 20.54698273, 22.46623219, 25.98451136, 31.43230549,
      42.54537896, 53.84673274, 66.63960707, 89.21748645, 138.2173146,
      245.0317673, 454.3420034, 911.3848661, 1869.712483};
    Mat_core_b2 = Mat_core_b()^2;
    Mat_core_h2 = Mat_core_h()^2;
    Mat_core_nu = Mat_core_h() / Mat_core_b();
    Mat_core_nu(0) = Mat_core_nu(1);
    Mat_core_nu_b2  = ListAlt[Mat_core_b2(), Mat_core_nu()] ;
    nu_core[] = InterpolationAkima[ SquNorm[$1] ]{ Mat_core_nu_b2() } ;
    dnudb2_core[] = dInterpolationAkima[SquNorm[$1]]{ Mat_core_nu_b2() } ;
    h_core[] = nu_core[$1] * $1 ;
    dhdb_core[] = TensorDiag[1,1,1] * nu_core[$1#1] +
      2*dnudb2_core[#1] * SquDyadicProduct[#1]  ;

    nu[ Core ]  = nu_core[$1];
    dhdb[ Core ] = dhdb_core[$1];
  Else
    nu[ Core ]  = 1. / (murCore * mu0);
  EndIf

  sigma[ Core ] = 1e6 / 10;
  sigma[ Ind ] = 5e7;

  Ns[ Ind ] = 1000 ; // number of turns in coil
  Sc[ Ind ] = SurfaceArea[] ; // area of coil cross section

  // Current density in each coil portion for a unit current (will be multiplied
  // by the actual total current in the coil)
  js0[ Ind ] = Ns[] / Sc[] * Vector[0, 0, -1];

  // For a correct definition of the voltage:
  CoefGeos[] = 1; // planar model, 1 meter thick
}

Constraint {
  { Name MagneticVectorPotential_2D;
    Case {
      { Region Surface_bn0; Value 0; }
      { Region Surface_Inf; Value 0; }
    }
  }
  { Name Current_2D;
    Case {
      If(Flag_FrequencyDomain)
        // Amplitude of the phasor is set to "Current"
        { Region Ind; Value Current; }
      Else
        // Time-dependent value is set to "Current * myModulation[]"
        { Region Ind; Value Current; TimeFunction myModulation[]; }
      EndIf
    }
  }
  { Name Voltage_2D;
    Case {
      { Region Core; Value 0; }
    }
  }
}

Include "Lib_Magnetodynamics2D_av_Cir.pro";

PostOperation {
  { Name dyn; NameOfPostProcessing Magnetodynamics2D_av;
    Operation {
      Print[ az, OnElementsOf Vol_Mag, File "az.pos" ];
      Print[ b, OnElementsOf Vol_Mag, File "b.pos" ];
      Print[ j, OnElementsOf Vol_Mag, File "j.pos" ];
      Print[ I, OnRegion Ind, File "I.txt", Format TimeTable,
        SendToServer "Output/Imposed current" ];
    }
  }
  { Name sta; NameOfPostProcessing Magnetostatics2D_a;
    Operation {
      Print[ az, OnElementsOf Vol_Mag, File "az.pos" ];
      Print[ b, OnElementsOf Vol_Mag, File "b.pos" ];
      Print[ j, OnElementsOf Vol_Mag, File "j.pos" ];
    }
  }
}

// Choose resolution, computation and post-operation in interactive mode
DefineConstant [
  R_ = {"Magnetodynamics2D_av", Name "GetDP/1ResolutionChoices", Visible 0},
  C_ = {"-solve -pos -bin", Name "GetDP/9ComputeCommand", Visible 0},
  P_ = {"dyn", Name "GetDP/2PostOperationChoices", Visible 0}
];
